Package: testit
Type: Package
Title: A Simple Package for Testing R Packages
Version: 0.13
Authors@R: c(
  person("Yihui", "Xie", role = c("aut", "cre"), email = "xie@yihui.name", comment = c(ORCID = "0000-0003-0645-5666")), 
  person("Steven", "Mortimer", role = "ctb", email="reportmort@gmail.com")
  )
Description: Provides two convenience functions assert() and test_pkg() to
    facilitate testing R packages.
License: GPL-3
URL: https://github.com/yihui/testit
BugReports: https://github.com/yihui/testit/issues
Suggests: rstudioapi, covr
RoxygenNote: 7.1.1
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2021-04-14 13:30:17 UTC; yihui
Author: Yihui Xie [aut, cre] (<https://orcid.org/0000-0003-0645-5666>),
  Steven Mortimer [ctb]
Maintainer: Yihui Xie <xie@yihui.name>
Repository: CRAN
Date/Publication: 2021-04-14 14:40:02 UTC
Built: R 4.4.0; ; 2026-05-12 01:28:36 UTC; unix
